# This code is part of KQCircuits
# Copyright (C) 2021 IQM Finland Oy
#
# This program is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
# License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
# warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with this program. If not, see
# https://www.gnu.org/licenses/gpl-3.0.html.
#
# The software distribution should follow IQM trademark policy for open-source software
# (meetiqm.com/developers/osstmpolicy). IQM welcomes contributions to the code. Please see our contribution agreements
# for individuals (meetiqm.com/developers/clas/individual) and organizations (meetiqm.com/developers/clas/organization).


from pathlib import Path
from kqcircuits.pya_resolver import pya
from kqcircuits.simulations.export.ansys.ansys_export import export_ansys
from kqcircuits.simulations.export.simulation_export import export_simulation_oas
from kqcircuits.simulations.single_xmons_full_chip_sim import SingleXmonsFullChipSim
from kqcircuits.util.export_helper import create_or_empty_tmp_directory, open_with_klayout_or_default_application

# Prepare output directory
dir_path = create_or_empty_tmp_directory(Path(__file__).stem + "_output")

launchers = True
port_size = 200
if launchers:
    port_size = 900

# Simulation parameters
sim_class = SingleXmonsFullChipSim  # pylint: disable=invalid-name
sim_parameters = {
    'use_ports': True,
    'launchers': launchers,  # True includes bonding pads and tapers, false includes only waveguides
    'use_test_resonators': True,  # True makes XS1, false makes XS2
    'n': 16,  # Reduce number of point in waveguide corners
    'port_size': port_size
}
export_parameters = {
    'path': dir_path
}

# Create simulation
simulations = [sim_class(pya.Layout(), **sim_parameters)]

# Export Ansys files
export_ansys(simulations, **export_parameters)

# Write and open oas file
open_with_klayout_or_default_application(export_simulation_oas(simulations, dir_path))
